{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit DDOSessionTests;

{$I DDObjectsTests.inc}

interface

uses
  SysUtils, TestFramework, DDOTestExtensions, DDOClient,
  uDDOSessionTest_Proxy, uDDOSession2Test_Proxy,
  uDDOSessionTest_Stub, uDDOSession2Test_Stub;

type
  TDDOSessionTests = class(TDDOBaseRemoteTest)
  private
    FProxy11,
    FProxy12: TDDOSessionTest;
    FProxy21,
    FProxy22: TDDOSession2Test;
    FRequester2: TDDORequester;
  protected
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure SessionTest;
    procedure PerformanceTest; override;
  end;

implementation

uses
  Windows;

type
  TDDOSessionTest = class(TDDOSessionTest_Stub)
  public
    procedure SetValue(value: String); override;
  end;

  TDDOSession2Test = class(TDDOSession2Test_Stub)
  public
    procedure GetValue(out value: String); override;
  end;

const
  CValue1 = 'clientvalue1';
  CValue2 = 'clientvalue1';
  CKey = 'value';

//----------------------------------------------------------------------------
//2006-12-15 00:09 N,SM:

procedure TDDOSessionTests.PerformanceTest;
var
  i: Integer;
begin
  for i := 1 to 500 do
    SessionTest;
end;

//----------------------------------------------------------------------------
//2006-12-15 00:09 N,SM:

procedure TDDOSessionTests.SessionTest;
var
  lString: String;
begin
  FProxy11.SetValue(CValue1);
  FProxy12.SetValue(CValue2);
  FProxy21.GetValue(lString);
  Assert(lString = CValue1);
  FProxy22.GetValue(lString);
  Assert(lString = CValue2);
end;

//----------------------------------------------------------------------------
//2006-12-15 00:09 N,SM:

procedure TDDOSessionTests.SetUp;
begin
  inherited;
  FRequester2 := TDDORequester.Create(nil);
  FProxy11 := CreateProxy(uDDOSessionTest_Proxy.TDDOSessionTest);
  FProxy12 := CreateProxy(uDDOSessionTest_Proxy.TDDOSessionTest, FRequester2);
  FProxy21 := CreateProxy(uDDOSession2Test_Proxy.TDDOSession2Test);
  FProxy22 := CreateProxy(uDDOSession2Test_Proxy.TDDOSession2Test, FRequester2);
  RegisterClass(TDDOSessionTest);
  RegisterClass(TDDOSession2Test);
end;

//----------------------------------------------------------------------------
//2006-12-15 00:09 N,SM:

procedure TDDOSessionTests.TearDown;
begin
  FreeAndNil(FRequester2);
  inherited;
end;

//----------------------------------------------------------------------------
//2007-01-10 10:13 N,SM:

procedure TDDOSessionTest.SetValue(value: String);
begin
  Session.Value[CKey] := value;
end;

//----------------------------------------------------------------------------
//2007-01-10 10:13 N,SM:

procedure TDDOSession2Test.GetValue(out value: String);
begin
  value := Session.Value[CKey];
end;

initialization
  TestFramework.RegisterTest('DDORemoteTests',
    TDDOSessionTests.Suite);

end.
