{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit DDOStatefullTests;

{$I DDObjectsTests.inc}

interface

uses
  SysUtils, TestFramework, DDOTestExtensions,
  uDDOStatefullTestClass_Proxy, uDDOStatefullTestClass_Stub;

type
  TDDOStatefullTests = class(TDDOBaseRemoteTest)
  private
    FClientA,
    FClientB: uDDOStatefullTestClass_Proxy.TDDOStatefullTestClass;
  protected
    procedure Setup; override;
  published
    procedure StatefullTest;
    procedure PerformanceTest; override;
    {$IFDEF MemoryTests}
    procedure MemoryUsageTest;
    {$ENDIF}
  end;

implementation

type
  TDDOStatefullTestClass = class(TDDOStatefullTestClass_Stub)
  private
    FStoredString: String;
  public
    procedure StoreString(DDOParam0: String); override;
    procedure GetString(out DDOParam0: String); override;
  end;

//----------------------------------------------------------------------------
//2006-07-02 03:55 N,SM:

procedure TDDOStatefullTestClass.GetString(out DDOParam0: String);
begin
  DDOParam0 := FStoredString;
end;

//----------------------------------------------------------------------------
//2006-07-02 03:55 N,SM:

procedure TDDOStatefullTestClass.StoreString(DDOParam0: String);
begin
  FStoredString := DDOParam0;
end;

//----------------------------------------------------------------------------
//2006-07-02 03:57 N,SM:

{$IFDEF MemoryTests}
procedure TDDOStatefullTests.MemoryUsageTest;
var
  i: integer;
begin
  for i := 1 to 10 do
    StatefullTest;
  ExpectedLeak := MemoryStatus.Allocated;
  for i := 1 to 100 do
    StatefullTest;
end;
{$ENDIF}

//----------------------------------------------------------------------------
//2006-07-02 04:03 N,SM:

procedure TDDOStatefullTests.PerformanceTest;
var
  i: Integer;
begin
  for i := 1 to 500 do
    StatefullTest;
end;

//----------------------------------------------------------------------------
//2006-07-02 04:03 N,SM:

procedure TDDOStatefullTests.Setup;
begin
  inherited;
  FClientA := CreateProxy(uDDOStatefullTestClass_Proxy.TDDOStatefullTestClass);
  FClientB := CreateProxy(uDDOStatefullTestClass_Proxy.TDDOStatefullTestClass);
  RegisterClass(TDDOStatefullTestClass);
end;

//----------------------------------------------------------------------------
//2006-07-02 03:56 N,SM:

procedure TDDOStatefullTests.StatefullTest;
const
  CClientValueA = 'string, client A';
  CClientValueB = 'string, client B';
var
  lData: String;
begin
  FClientA.StoreString(CClientValueA);
  FClientB.StoreString(CClientValueB);
  FClientA.GetString(lData);
  CheckEqualsString(CClientValueA, lData);
  FClientB.GetString(lData);
  CheckEqualsString(CClientValueB, lData);
end;

initialization
  TestFramework.RegisterTest('DDORemoteTests',
    TDDOStatefullTests.Suite);

end.
