{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  This software is distributed as shareware. It is NOT free. You may use     }
{  the software for a trial period of thirty (30) days, at no cost to you,    }
{  to determine if it fits your needs. If you decide to use the software,     }
{  you shall register it and pay the applicable registration fee.             }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

program NameServer;

{$APPTYPE CONSOLE}

uses
  Classes, SysUtils, Windows, DDONameServer;

type
  TDDONameServerDemo = class(TObject)
  private
    FNameServer: TDDONameServer;
  protected
    procedure RegisterClasses;
    procedure Lookup(Sender: TObject; const ClassName: String;
      var Host: String; var Port: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
  end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:
//2006-02-27 R,SM:

constructor TDDONameServerDemo.Create;
begin
  inherited;
  FNameServer := TDDONameServer.Create(nil);
  FNameServer.OnLookup := Lookup;
  RegisterClasses;
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:
//2006-02-27 R,SM:

destructor TDDONameServerDemo.Destroy;
begin
  FreeAndNil(FNameServer);
  inherited;
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:
//2006-02-27 R,SM:

procedure TDDONameServerDemo.Execute;
begin
  FNameServer.Active := True;
  WriteLn('DDONameServer started on port ' + IntToStr(FNameServer.Port));
  WriteLn('>> Press ESC to shutdown <<' + #13#10);
  while GetAsyncKeyState(VK_ESCAPE) = 0 do
  begin
  {$IFNDEF VER130}
    CheckSynchronize;
  {$ENDIF}
    Sleep(0);
  end;
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:
//2006-02-27 R,SM:

procedure TDDONameServerDemo.Lookup(Sender: TObject;
  const ClassName: String; var Host: String; var Port: Integer);
begin
  WriteLn('Lookup for ''' + ClassName + ''' received');
  WriteLn('  Result: ' + Host + ':' + IntToStr(Port) + #13#10);
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:
//2006-02-27 R,SM:

procedure TDDONameServerDemo.RegisterClasses;
begin
  FNameServer.RegisterClass('TTestClassA', 'localhost', 6667);
  FNameServer.RegisterClass('TTestClassB', 'localhost', 6668);
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:
//2006-02-27 R,SM:

begin
  with TDDONameServerDemo.Create do
  try
    Execute;
  finally
    Free;
  end;
end.
