{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit uDDOServerForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, StdCtrls, Spin, DDOServer, DDODocument, ComCtrls,
  ExtCtrls, DDOBase, DDODataGrid, DDOProgressBar, DDOUtils, DDOSystem;

type
  TMainForm = class(TForm)
    DDOListener1: TDDOListener;
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    GridUtil: TDDODataGrid;
    GridThreads: TDDODataGrid;
    GridCalls: TDDODataGrid;
    DDOProgressBar1: TDDOProgressBar;
    ListView1: TListView;
    Label1: TLabel;
    EditPort: TSpinEdit;
    SpeedButtonStart: TSpeedButton;
    SpeedButtonUpdate: TSpeedButton;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DDOListener1Activate(Sender: TObject);
    procedure DDOListener1ClientException(
      Sender: TObject; E: Exception; Method: String);
    procedure DDOListener1RequestProcessed(
      Sender: TObject; Result: TDDODocument);
    procedure GridCallsGetData(Sender: TObject; Index: Integer;
      var Value: Integer);
    procedure GridUtilGetData(Sender: TObject; Index: Integer;
      var Value: Integer);
    procedure GridThreadsGetData(Sender: TObject; Index: Integer;
      var Value: Integer);
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure SpeedButtonStartClick(Sender: TObject);
    procedure SpeedButtonUpdateClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FProcessTimes: TDDOProcessTimes;
    FRequests, FLastValue: Integer;
    procedure ReadCmdLineParams;
    procedure Log(AMessage: String = '');
    procedure UpdateGUI;
    procedure SendQuote(Stub: TDDOStubBase; UserData: Integer;
      var Continue: Boolean);
  end;

var
  MainForm: TMainForm;

implementation

uses
  uDemoClass_Stub, uDemoLogIn_Stub;

{$R *.dfm}

const
  CStarted = 'Started (port %d)';
  CStopped = 'Stopped';
  CException = '%s (%s) %s';
  CFileName = 'Quotes.txt';
  CPassword = 'password';
  CKeyLoggedIn = 'loggedin';
  CKeyUserName = 'username';
  CKeyData = 'data';

type
  TDemoClass = class(TDemoClass_ActiveStub)
  private
    FQuotes: TStrings;
    procedure ValidateLoggedIn;
  public
    procedure GetQuote(out Quote: String); override;
    procedure GetTime(out Time: TDateTime); override;
    procedure GetVarParam(var Param: String); override;
    procedure AsyncInvoke(ID: Integer; out Data: String); override;
    procedure GetSessionData(out Data: String); override;
    procedure SetSessionData(Data: String); override;
    procedure Initialize; override;
    destructor Destroy; override;
  end;

  TDemoLogIn = class(TDemoLogIn_Stub)
  public
    procedure LogIn(User: String; Password: String); override;
    procedure LogOff; override;
  end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.SpeedButtonStartClick(Sender: TObject);
begin
  try
    if not DDOListener1.Active then
      DDOListener1.Port := EditPort.Value;
    DDOListener1.Active := not DDOListener1.Active;
  finally
    UpdateGUI;
  end;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.FormCreate(Sender: TObject);
begin
  DDOListener1.RegisterClass(TDemoClass);
  DDOListener1.RegisterClass(TDemoLogIn);
  FProcessTimes := TDDOProcessTimes.Create;
  ListView1.DoubleBuffered := True;
  Constraints.MinWidth := Width;
  Constraints.MinHeight := Height;
  ReadCmdLineParams;
  UpdateGUI;
end;

//----------------------------------------------------------------------------
//2006-02-20 N,SM:

procedure TMainForm.ReadCmdLineParams;
  function nGetValue(Switch: String; Default: Integer): Integer;
  var
    i: Integer;
  begin
    Result := Default;
    for i := 0 to ParamCount -2 do
      if ParamStr(i) = Switch then
        Result := StrToIntDef(ParamStr(i+1), Default);
  end;
begin
  DDOListener1.Port := nGetValue('-p', DDOListener1.Port);
  DDOListener1.Active := FindCmdLineSwitch('a', ['-'], False);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  DDOListener1.Active := False;
  FreeAndNil(FProcessTimes);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.UpdateGUI;
begin
  SpeedButtonStart.Down := DDOListener1.Active;
  EditPort.Enabled := not DDOListener1.Active;
  EditPort.Value := DDOListener1.Port;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.Log(AMessage: String = '');
begin
  if AMessage = EmptyStr then
    if DDOListener1.Active then
      AMessage := Format(CStarted, [DDOListener1.Port])
    else
      AMessage := CStopped;
  StatusBar1.SimpleText := AMessage
    + ': ' + IntToStr(DDOListener1.Requests);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.DDOListener1Activate(Sender: TObject);
begin
  Log;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.DDOListener1ClientException(Sender: TObject;
  E: Exception; Method: String);
begin
  Log(Format(CException, [E.ClassName, E.Message, Method]));
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.DDOListener1RequestProcessed(Sender: TObject;
  Result: TDDODocument);
begin
  InterlockedIncrement(FRequests);
  Log;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.GridCallsGetData(Sender: TObject;
  Index: Integer; var Value: Integer);
begin
  Value := (FRequests - FLastValue) * 1000 div GridCalls.Interval;
  FLastValue := FRequests;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.GridUtilGetData(Sender: TObject;
  Index: Integer; var Value: Integer);
begin
  DDOProgressBar1.Value := Trunc(FProcessTimes.CPUUsage);
  Value := Trunc(DDOProgressBar1.Value);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.GridThreadsGetData(Sender: TObject;
  Index: Integer; var Value: Integer);
begin
  Value := DDOListener1.ActiveThreads;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.ListView1Data(Sender: TObject; Item: TListItem);
begin
  with TDDOServerUtils.GetStubTracker, Item do
  begin
    Caption := Name[Index];
    SubItems.Add(IntToStr(Calls[Index]));
    SubItems.Add(IntToStr(Last[Index]));
    SubItems.Add(IntToStr(Min[Index]));
    SubItems.Add(IntToStr(Max[Index]));
  end;
end;

//----------------------------------------------------------------------------
//2006-02-27 N,SM:

procedure TMainForm.SpeedButtonUpdateClick(Sender: TObject);
begin
  ListView1.Items.Count := TDDOServerUtils.GetStubTracker.Count;
  ListView1.Invalidate;
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.AsyncInvoke(ID: Integer; out Data: String);
begin
  Data := 'Some Data for ID ' + IntToStr(ID);
  Sleep(Random(5000) + 5000);  // simulate some "lenghty" process
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

destructor TDemoClass.Destroy;
begin
  FreeAndNil(FQuotes);
  inherited;
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.GetQuote(out Quote: String);
begin
  // due to +1 an exception will be raised from time to time
  // this is intentional to show how DDObjects handles exceptions
  Quote := FQuotes[Random(FQuotes.Count +1)];
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.GetSessionData(out Data: String);
begin
  ValidateLoggedIn;
  Data := Session.Value[CKeyUserName];
  if Session.Exists(CKeyData) then
    Data := Data + ': ' + Session.Value[CKeyData];
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.GetTime(out Time: TDateTime);
begin
  Time := Now;
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.GetVarParam(var Param: String);
begin
  Param := Param + Param;
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.Initialize;
begin
  inherited;
  FQuotes := TStringList.Create;
  FQuotes.LoadFromFile(CFileName);
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.SetSessionData(Data: String);
begin
  ValidateLoggedIn;
  Session.Value[CKeyData] := Data;
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoClass.ValidateLoggedIn;
begin
  if not Session.Exists(CKeyLoggedIn)
  or not Session.Value[CKeyLoggedIn] then
    raise Exception.Create('Not logged in');
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoLogIn.LogIn(User, Password: String);
begin
  if Password <> CPassword then
    raise Exception.Create('Invalid password');
  Session.Value[CKeyLoggedIn] := true;
  Session.Value[CKeyUserName] := User;
end;

//----------------------------------------------------------------------------
//2007-03-06 N,SM:

procedure TDemoLogIn.LogOff;
begin
  Session.Value[CKeyLoggedIn] := false;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.SendQuote(Stub: TDDOStubBase;
  UserData: Integer; var Continue: Boolean);
var
  lDemoClass: TDemoClass;
begin
  Assert(Stub is TDemoClass);
  lDemoClass := TDemoClass(Stub);
  lDemoClass.Notification(Now,
    lDemoClass.FQuotes[Random(lDemoClass.FQuotes.Count)], nil);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TMainForm.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled := False;
  try
    DDOListener1.EnumStubs(TDemoClass, SendQuote, 0);
  finally
    Timer1.Enabled := True;
  end;
end;

end.
